@@ -1,7 +1,7 @@
 import functools
-from typing import Any, Dict, Type
+from typing import Any, Dict, Type, Union, List, TypeVar, Callable
 
-from mongoengine import Document, EmbeddedDocument, fields
+from mongoengine import Document, EmbeddedDocument, fields as mongoengine_fields
 from mongoengine.base import TopLevelDocumentMetaclass
 
 from aggify.compiler import F, Match, Q, Operators, Cond  # noqa keep
@@ -12,8 +12,9 @@ from aggify.exceptions import (
     InvalidEmbeddedField,
     OutStageError,
     InvalidArgument,
+    InvalidProjection,
 )
-from aggify.types import QueryParams
+from aggify.types import QueryParams, CollectionType
 from aggify.utilty import (
     to_mongo_positive_index,
     check_fields_exist,
@@ -23,8 +24,10 @@ from aggify.utilty import (
     get_db_field,
 )
 
+AggifyType = TypeVar("AggifyType", bound=Callable[..., "Aggify"])
 
-def last_out_stage_check(method):
+
+def last_out_stage_check(method: AggifyType) -> AggifyType:
     """Check if the last stage is $out or not
 
     This decorator check if the last stage is $out or not
@@ -34,7 +37,7 @@ def last_out_stage_check(method):
     @functools.wraps(method)
     def decorator(*args, **kwargs):
         try:
-            if last_is_out := bool(args[0].pipelines[-1].get("$out")):
+            if bool(args[0].pipelines[-1].get("$out")):
                 raise OutStageError(method.__name__)
         except IndexError:
             return method(*args, **kwargs)
@@ -52,28 +55,87 @@ class Aggify:
         Args:
             base_model: The base model class.
         """
-        self.base_model = base_model
-        self.pipelines: list[dict[str, dict | Any]] = []
+        # Create a separate copy of the main class for safety and flexibility
+        self.base_model = type(
+            "Aggify_base_model", base_model.__bases__, dict(base_model.__dict__)
+        )
+        self.pipelines: List[Dict[str, Union[dict, Any]]] = []
         self.start = None
         self.stop = None
         self.q = None
 
+    def __iter__(self):
+        # Return a generator or iterator for the data you want to represent as a list
+        return iter(self.pipelines)
+
     @last_out_stage_check
     def project(self, **kwargs: QueryParams) -> "Aggify":
-        self.pipelines.append({"$project": kwargs})
+        """
+        Adjusts the base model's fields based on the given keyword arguments.
+
+        Fields to be retained are set to 1 in kwargs.
+        Fields to be deleted are set to 0 in kwargs, except for _id which is controlled by the delete_id flag.
+
+        Args:
+            **kwargs: Fields to be retained or removed.
+                      For example: {"field1": 1, "field2": 0}
+                      _id field behavior: {"id": 0} means delete _id.
+
+        Returns:
+            Aggify: Returns an instance of the Aggify class for potential method chaining.
+        """
+
+        if all([i in kwargs.values() for i in [0, 1]]):
+            raise InvalidProjection()
+
+        # Extract fields to keep and check if _id should be deleted
+        to_keep_values = {"id"}
+        delete_id = kwargs.get("id") is not None
+        projection = {}
+
+        # Add missing fields to the base model
+        for key, value in kwargs.items():
+            if value == 1:
+                to_keep_values.add(key)
+            elif key not in self.base_model._fields and isinstance(  # noqa
+                kwargs[key], (str, dict)
+            ):
+                to_keep_values.add(key)
+                self.base_model._fields[key] = mongoengine_fields.IntField()  # noqa
+            projection[get_db_field(self.base_model, key)] = value  # noqa
+            if value == 0:
+                del self.base_model._fields[key]  # noqa
+
+        # Remove fields from the base model, except the ones in to_keep_values and possibly _id
+        if to_keep_values != {"id"}:
+            keys_for_deletion = self.base_model._fields.keys() - to_keep_values  # noqa
+            if delete_id:
+                keys_for_deletion.add("id")
+            for key in keys_for_deletion:
+                del self.base_model._fields[key]  # noqa
+        # Append the projection stage to the pipelines
+        self.pipelines.append({"$project": projection})
+
+        # Return the instance for method chaining
         return self
 
     @last_out_stage_check
-    def group(self, expression: str | None = "_id") -> "Aggify":
-        expression = f"${expression}" if expression else None
+    def group(self, expression: Union[str, None] = "id") -> "Aggify":
+        if expression:
+            try:
+                expression = "$" + self.get_field_name_recursively(expression)
+            except InvalidField:
+                pass
         self.pipelines.append({"$group": {"_id": expression}})
         return self
 
     @last_out_stage_check
-    def order_by(self, *fields: str | list[str]) -> "Aggify":
+    def order_by(self, *order_fields: Union[str, List[str]]) -> "Aggify":
         sort_dict = {
-            field.replace("-", ""): -1 if field.startswith("-") else 1
-            for field in fields
+            get_db_field(self.base_model, field.replace("-", "")): -1
+            if field.startswith("-")
+            else 1
+            for field in order_fields
         }
         self.pipelines.append({"$sort": sort_dict})
         return self
@@ -85,8 +147,7 @@ class Aggify:
 
     @last_out_stage_check
     def add_fields(self, **fields) -> "Aggify":  # noqa
-        """
-        Generates a MongoDB addFields pipeline stage.
+        """Generates a MongoDB addFields pipeline stage.
 
         Args:
             fields: A dictionary of field expressions and values.
@@ -97,37 +158,49 @@ class Aggify:
         add_fields_stage = {"$addFields": {}}
 
         for field, expression in fields.items():
+            field = field.replace("__", ".")
             if isinstance(expression, str):
                 add_fields_stage["$addFields"][field] = {"$literal": expression}
             elif isinstance(expression, F):
                 add_fields_stage["$addFields"][field] = expression.to_dict()
+            elif isinstance(expression, (list, dict)):
+                add_fields_stage["$addFields"][field] = expression
             elif isinstance(expression, Cond):
                 add_fields_stage["$addFields"][field] = dict(expression)
+            elif isinstance(expression, Q):
+                add_fields_stage["$addFields"][field] = convert_match_query(
+                    dict(expression)
+                )["$match"]
             else:
-                raise AggifyValueError([str, F], type(expression))
+                raise AggifyValueError([str, F, list], type(expression))
+            # TODO: Should be checked if new field is embedded, create embedded field.
+            self.base_model._fields[  # noqa
+                field.replace("$", "")
+            ] = mongoengine_fields.IntField()
 
         self.pipelines.append(add_fields_stage)
         return self
 
     @last_out_stage_check
-    def filter(self, arg: Q | None = None, **kwargs: QueryParams) -> "Aggify":
+    def filter(
+        self, arg: Union[Q, None] = None, **kwargs: Union[QueryParams, F, list]
+    ) -> "Aggify":
         """
         # TODO: missing docs
         """
         if arg is not None and isinstance(arg, Q) is not True:
             raise AggifyValueError([Q, None], type(arg))
 
-        if arg is None:
-            self.q = kwargs
-            self.__to_aggregate(self.q)
-            self.pipelines = self.__combine_sequential_matches()
-
         if isinstance(arg, Q):
             self.pipelines.append(dict(arg))
 
+        self.q = kwargs
+        self.__to_aggregate(self.q)
+        self.pipelines = self.__combine_sequential_matches()
+
         return self
 
-    def out(self, coll: str, db: str | None = None) -> "Aggify":
+    def out(self, coll: str, db: Union[str, None] = None) -> "Aggify":
         """Write the documents returned by the aggregation pipeline into specified collection.
 
         Starting in MongoDB 4.4, you can specify the output database.
@@ -168,16 +241,12 @@ class Aggify:
         self.pipelines.append(stage)
         return self
 
-    def __to_aggregate(self, query: dict[str, Any]) -> None:
+    def __to_aggregate(self, query: Dict[str, Any]) -> None:
         """
         Builds the pipelines list based on the query parameters.
         """
-        skip_list = []
 
         for key, value in query.items():
-            if key in skip_list:
-                continue
-
             # Split the key to access the field information.
             split_query = key.split("__")
 
@@ -191,7 +260,7 @@ class Aggify:
                 or "document_type_obj"
                 not in join_field.__dict__  # Check whether this field is a join field or not.
                 or issubclass(
-                    join_field.document_type, EmbeddedDocument
+                    join_field.document_type, EmbeddedDocument  # noqa
                 )  # Check whether this field is embedded field or not
                 or len(split_query) == 1
                 or (len(split_query) == 2 and split_query[1] in Operators.ALL_OPERATORS)
@@ -210,7 +279,6 @@ class Aggify:
                 matches = []
                 for k, v in query.items():
                     if k.split("__")[0] == split_query[0]:
-                        skip_list.append(k)
                         _key = k.replace("__", ".", 1)
                         match = self.__match({_key: v}).get("$match")
                         if match != {}:
@@ -227,7 +295,7 @@ class Aggify:
                 self.pipelines.extend([{"$match": match} for match in matches])
 
     @last_out_stage_check
-    def __getitem__(self, index: slice | int) -> "Aggify":
+    def __getitem__(self, index: Union[slice, int]) -> "Aggify":
         """
         # TODO: missing docs
         """
@@ -241,7 +309,10 @@ class Aggify:
 
     @last_out_stage_check
     def unwind(
-        self, path: str, include_index_array: str | None = None, preserve: bool = False
+        self,
+        path: str,
+        include_array_index: Union[str, None] = None,
+        preserve: bool = False,
     ) -> "Aggify":
         """Generates a MongoDB unwind pipeline stage.
 
@@ -250,7 +321,7 @@ class Aggify:
               To specify a field path, prefix the field name with a dollar sign $
               and enclose in quotes.
 
-            include_index_array: The name of a new field to hold the array index of the element.
+            include_array_index: The name of a new field to hold the array index of the element.
               The name cannot start with a dollar sign $.
 
             preserve: Whether to preserve null and empty arrays.
@@ -274,32 +345,22 @@ class Aggify:
 
         docs: https://www.mongodb.com/docs/manual/reference/operator/aggregation/unwind/
         """
-
-        if include_index_array is None and preserve is False:
-            self.pipelines.append({"$unwind": f"${path}"})
-            return self
-        self.pipelines.append(
-            {
-                "$unwind": {
-                    "path": f"${path}",
-                    "includeArrayIndex": include_index_array,
-                    "preserveNullAndEmptyArrays": preserve,
-                }
-            }
-        )
+        path = self.get_field_name_recursively(path)
+        if include_array_index is None and preserve is False:
+            unwind_stage = {"$unwind": f"${path}"}
+        else:
+            unwind_stage = {"$unwind": {"path": f"${path}"}}
+            if preserve:
+                unwind_stage["$unwind"]["preserveNullAndEmptyArrays"] = preserve
+            if include_array_index:
+                unwind_stage["$unwind"][
+                    "includeArrayIndex"
+                ] = include_array_index.replace("$", "")
+        self.pipelines.append(unwind_stage)
         return self
 
-    def aggregate(self):
-        """
-        Returns the aggregated results.
-
-        Returns:
-            The aggregated results.
-        """
-        return self.base_model.objects.aggregate(*self.pipelines)  # type: ignore
-
     def annotate(
-        self, annotate_name: str, accumulator: str, f: str | dict | F | int
+        self, annotate_name: str, accumulator: str, f: Union[Union[str, Dict], F, int]
     ) -> "Aggify":
         """
         Annotate a MongoDB aggregation pipeline with a new field.
@@ -308,7 +369,7 @@ class Aggify:
         Args:
             annotate_name (str): The name of the new annotated field.
             accumulator (str): The aggregation accumulator operator (e.g., "$sum", "$avg").
-            f (str | dict | F | int): The value for the annotated field.
+            f (Union[str, Dict] | F | int): The value for the annotated field.
 
         Returns:
             self.
@@ -321,31 +382,49 @@ class Aggify:
         """
 
         # Some of the accumulator fields might be false and should be checked.
+        # noinspection SpellCheckingInspection
         aggregation_mapping: Dict[str, Type] = {
-            "sum": (fields.FloatField(), "$sum"),
-            "avg": (fields.FloatField(), "$avg"),
-            "stdDevPop": (fields.FloatField(), "$stdDevPop"),
-            "stdDevSamp": (fields.FloatField(), "$stdDevSamp"),
-            "push": (fields.ListField(), "$push"),
-            "addToSet": (fields.ListField(), "$addToSet"),
-            "count": (fields.IntField(), "$count"),
-            "first": (fields.EmbeddedDocumentField(fields.EmbeddedDocument), "$first"),
-            "last": (fields.EmbeddedDocumentField(fields.EmbeddedDocument), "$last"),
-            "max": (fields.DynamicField(), "$max"),
-            "accumulator": (fields.DynamicField(), "$accumulator"),
-            "min": (fields.DynamicField(), "$min"),
-            "median": (fields.DynamicField(), "$median"),
-            "mergeObjects": (fields.DictField(), "$mergeObjects"),
-            "top": (fields.EmbeddedDocumentField(fields.EmbeddedDocument), "$top"),
+            "sum": (mongoengine_fields.FloatField(), "$sum"),
+            "avg": (mongoengine_fields.FloatField(), "$avg"),
+            "stdDevPop": (mongoengine_fields.FloatField(), "$stdDevPop"),
+            "stdDevSamp": (mongoengine_fields.FloatField(), "$stdDevSamp"),
+            "push": (mongoengine_fields.ListField(), "$push"),
+            "addToSet": (mongoengine_fields.ListField(), "$addToSet"),
+            "count": (mongoengine_fields.IntField(), "$count"),
+            "first": (
+                mongoengine_fields.EmbeddedDocumentField(
+                    mongoengine_fields.EmbeddedDocument
+                ),
+                "$first",
+            ),
+            "last": (
+                mongoengine_fields.EmbeddedDocumentField(
+                    mongoengine_fields.EmbeddedDocument
+                ),
+                "$last",
+            ),
+            "max": (mongoengine_fields.DynamicField(), "$max"),
+            "accumulator": (mongoengine_fields.DynamicField(), "$accumulator"),
+            "min": (mongoengine_fields.DynamicField(), "$min"),
+            "median": (mongoengine_fields.DynamicField(), "$median"),
+            "mergeObjects": (mongoengine_fields.DictField(), "$mergeObjects"),
+            "top": (
+                mongoengine_fields.EmbeddedDocumentField(
+                    mongoengine_fields.EmbeddedDocument
+                ),
+                "$top",
+            ),
             "bottom": (
-                fields.EmbeddedDocumentField(fields.EmbeddedDocument),
+                mongoengine_fields.EmbeddedDocumentField(
+                    mongoengine_fields.EmbeddedDocument
+                ),
                 "$bottom",
             ),
-            "topN": (fields.ListField(), "$topN"),
-            "bottomN": (fields.ListField(), "$bottomN"),
-            "firstN": (fields.ListField(), "$firstN"),
-            "lastN": (fields.ListField(), "$lastN"),
-            "maxN": (fields.ListField(), "$maxN"),
+            "topN": (mongoengine_fields.ListField(), "$topN"),
+            "bottomN": (mongoengine_fields.ListField(), "$bottomN"),
+            "firstN": (mongoengine_fields.ListField(), "$firstN"),
+            "lastN": (mongoengine_fields.ListField(), "$lastN"),
+            "maxN": (mongoengine_fields.ListField(), "$maxN"),
         }
 
         try:
@@ -369,19 +448,18 @@ class Aggify:
         else:
             if isinstance(f, str):
                 try:
-                    self.get_model_field(self.base_model, f)
-                    value = f"${f}"
+                    value = f"${self.get_field_name_recursively(f)}"
                 except InvalidField:
                     value = f
             else:
                 value = f
 
         # Determine the data type based on the aggregation operator
-        self.pipelines[-1]["$group"] |= {annotate_name: {acc: value}}
+        self.pipelines[-1]["$group"].update({annotate_name: {acc: value}})
         self.base_model._fields[annotate_name] = field_type  # noqa
         return self
 
-    def __match(self, matches: dict[str, Any]):
+    def __match(self, matches: Dict[str, Any]):
         """
         Generates a MongoDB match pipeline stage.
 
@@ -391,12 +469,12 @@ class Aggify:
         Returns:
             A MongoDB match pipeline stage.
         """
-        return Match(matches, self.base_model).compile(self.pipelines)
+        return Match(matches, self.base_model).compile(self.pipelines)  # noqa
 
     @staticmethod
     def __lookup(
         from_collection: str, local_field: str, as_name: str, foreign_field: str = "_id"
-    ) -> dict[str, dict[str, str]]:
+    ) -> Dict[str, Dict[str, str]]:
         """
         Generates a MongoDB lookup pipeline stage.
 
@@ -418,7 +496,7 @@ class Aggify:
             }
         }
 
-    def __combine_sequential_matches(self) -> list[dict[str, dict | Any]]:
+    def __combine_sequential_matches(self) -> List[Dict[str, Union[dict, Any]]]:
         merged_pipeline = []
         match_stage = None
 
@@ -440,15 +518,49 @@ class Aggify:
 
         return merged_pipeline
 
+    # check_fields_exist(self.base_model, let)  # noqa
+    def get_field_name_recursively(self, field: str) -> str:
+        """
+        Recursively fetch the field name by following the hierarchy indicated by the field parameter.
+
+        The function traverses the field hierarchy indicated by double underscores in the field parameter.
+        At each level, it checks if the field exists and then fetches the database field name for it.
+        The entire hierarchy is then joined using a dot (.) separator.
+
+        Parameters:
+        - field (str): A string indicating the hierarchy of fields, separated by double underscores.
+
+        Returns:
+        - str: A dot-separated string representing the full path to the field in the database.
+        """
+
+        field_name = []
+        prev_base = self.base_model
+
+        # Split the field based on double underscores and process each item
+        for index, item in enumerate(field.split("__")):
+            # Ensure the field exists at the current level of hierarchy
+            check_fields_exist(prev_base, [item])  # noqa
+
+            # Append the database field name to the field_name list
+            field_name.append(get_db_field(prev_base, item))
+
+            # Move to the next level in the model hierarchy
+            prev_base = self.get_model_field(prev_base, item)
+            prev_base = prev_base.__dict__.get("document_type_obj", prev_base)
+
+        # Join the entire hierarchy using dots and return
+        return ".".join(field_name)
+
     @last_out_stage_check
     def lookup(
         self,
-        from_collection: Document,
+        from_collection: CollectionType,
         as_name: str,
-        query: list[Q] | Q | None = None,
-        let: list[str] | None = None,
-        local_field: str | None = None,
-        foreign_field: str | None = None,
+        query: Union[List[Q], Union[Q, None], List["Aggify"]] = None,
+        let: Union[List[str], None] = None,
+        local_field: Union[str, None] = None,
+        foreign_field: Union[str, None] = None,
     ) -> "Aggify":
         """
         Generates a MongoDB lookup pipeline stage.
@@ -456,17 +568,22 @@ class Aggify:
         Args:
             from_collection (Document): The document representing the collection to perform the lookup on.
             as_name (str): The name of the new field to create.
-            query (list[Q] | Q | None, optional): List of desired queries with Q function or a single query.
-            let (list[str] | None, optional): The local field(s) to join on. If provided, localField and foreignField are not used.
-            local_field (str | None, optional): The local field to join on when let is not provided.
-            foreign_field (str | None, optional): The foreign field to join on when let is not provided.
+            query (list[Q] | Union[Q, None], optional): List of desired queries with Q function or a single query.
+            let (Union[List[str], None], optional): The local field(s) to join on. If provided,
+            localField and foreignField are not used.
+            local_field (Union[str, None], optional): The local field to join on when `let` is not provided.
+            foreign_field (Union[str, None], optional): The foreign field to join on when `let` is not provided.
+            let (Union[List[str], None], optional): The local field(s) to join on. If provided,
+            localField and foreignField are not used.
+            local_field (Union[str, None], optional): The local field to join on when let not provided.
+            foreign_field (Union[str, None], optional): The foreign field to join on when let not provided.
 
         Returns:
             Aggify: An instance of the Aggify class representing a MongoDB lookup pipeline stage.
         """
 
         lookup_stages = []
-        check_field_exists(self.base_model, as_name)
+        check_field_exists(self.base_model, as_name)  # noqa
         from_collection_name = from_collection._meta.get("collection")  # noqa
 
         if not let and not (local_field and foreign_field):
@@ -474,26 +591,25 @@ class Aggify:
                 expected_list=[["local_field", "foreign_field"], "let"]
             )
         elif not let:
-            if not (local_field and foreign_field):
-                raise InvalidArgument(expected_list=["local_field", "foreign_field"])
             lookup_stage = {
                 "$lookup": {
                     "from": from_collection_name,
                     "localField": get_db_field(self.base_model, local_field),  # noqa
                     "foreignField": get_db_field(
-                        from_collection, foreign_field
-                    ),  # noqa
+                        from_collection, foreign_field  # noqa
+                    ),
                     "as": as_name,
                 }
             }
         else:
             if not query:
                 raise InvalidArgument(expected_list=["query"])
-            check_fields_exist(self.base_model, let)  # noqa
+
             let_dict = {
-                field: f"${get_db_field(self.base_model, field)}"
-                for field in let  # noqa
+                field: f"${get_db_field(self.base_model, self.get_field_name_recursively(field))}"  # noqa
+                for field in let
             }
+
             for q in query:
                 # Construct the match stage for each query
                 if isinstance(q, Q):
@@ -529,12 +645,12 @@ class Aggify:
         return self
 
     @staticmethod
-    def get_model_field(model: Type[Document], field: str) -> fields:
+    def get_model_field(model: Type[Document], field: str) -> mongoengine_fields:
         """
         Get the field definition of a specified field in a MongoDB model.
 
         Args:
-            model (Document): The MongoDB model.
+            model (CollectionType): The MongoDB model.
             field (str): The name of the field to retrieve.
 
         Returns:
@@ -561,7 +677,7 @@ class Aggify:
         Raises:
             InvalidEmbeddedField: If the specified embedded field is not found or is not of the correct type.
         """
-        model_field = self.get_model_field(self.base_model, embedded_field)
+        model_field = self.get_model_field(self.base_model, embedded_field)  # noqa
 
         if not hasattr(model_field, "document_type") or not issubclass(
             model_field.document_type, EmbeddedDocument
@@ -572,14 +688,14 @@ class Aggify:
 
     @last_out_stage_check
     def replace_root(
-        self, *, embedded_field: str, merge: dict | None = None
+        self, *, embedded_field: str, merge: Union[Dict, None] = None
     ) -> "Aggify":
         """
         Replace the root document in the aggregation pipeline with a specified embedded field or a merged result.
 
         Args:
             embedded_field (str): The name of the embedded field to use as the new root.
-            merge (dict | None, optional): A dictionary for merging with the new root. Default is None.
+            merge (Union[Dict, None], optional): A dictionary for merging with the new root. Default is None.
 
         Returns:
             Aggify: The modified Aggify instance.
@@ -600,14 +716,14 @@ class Aggify:
 
     @last_out_stage_check
     def replace_with(
-        self, *, embedded_field: str, merge: dict | None = None
+        self, *, embedded_field: str, merge: Union[Dict, None] = None
     ) -> "Aggify":
         """
         Replace the root document in the aggregation pipeline with a specified embedded field or a merged result.
 
         Args:
             embedded_field (str): The name of the embedded field to use as the new root.
-            merge (dict | None, optional): A dictionary for merging with the new root. Default is None.
+            merge (Union[Dict, None], optional): A dictionary for merging with the new root. Default is None.
 
         Returns:
             Aggify: The modified Aggify instance.
@@ -625,3 +741,61 @@ class Aggify:
         self.pipelines.append(new_root)
 
         return self
+
+    @last_out_stage_check
+    def redact(self, value1, condition, value2, then_value, else_value):
+        """
+        Add a $redact stage to the pipeline based on the provided conditions.
+
+        The $redact stage restricts the contents of the returned documents
+        based on the condition specified. This method assists in building
+        the $redact stage for a MongoDB aggregation pipeline.
+
+        Parameters:
+        - value1 (str): The first value to compare in the condition.
+        - condition (str): The MongoDB comparison operator (e.g., "$eq", "$lt").
+        - value2 (str): The second value to compare in the condition.
+        - then_value (str): The action to take if the condition is True.
+                            Expected values: "$DESCEND", "$PRUNE", "$KEEP".
+        - else_value (str): The action to take if the condition is False.
+                            Expected values: "$DESCEND", "$PRUNE", "$KEEP".
+
+        Returns:
+        - self: Returns the instance of the class to allow chaining.
+
+        Raises:
+        - InvalidArgument: If then_value or else_value are not in the expected list.
+        """
+
+        # List of valid redaction values
+        redact_values = ["DESCEND", "PRUNE", "KEEP"]
+
+        def clean_then_else(_then_value, _else_value):
+            """
+            Helper function to sanitize then_value and else_value.
+
+            Strips the dollar sign and converts values to uppercase.
+            """
+            return (
+                _then_value.replace("$", "").upper(),
+                _else_value.replace("$", "").upper(),
+            )
+
+        # Clean the provided then_value and else_value
+        then_value, else_value = clean_then_else(then_value, else_value)
+
+        # Check if the cleaned values are in the valid list
+        if then_value not in redact_values or else_value not in redact_values:
+            raise InvalidArgument(expected_list=redact_values)
+
+        # Construct the $redact stage with the provided condition
+        stage = {
+            "$redact": dict(
+                Cond(value1, condition, value2, f"$${then_value}", f"$${else_value}")
+            )
+        }
+
+        # Append the constructed stage to the pipelines list
+        self.pipelines.append(stage)
+
+        return self
