@@ -1,4 +1,4 @@
-from typing import Any, Type
+from typing import Any, Type, Union, Dict
 
 from mongoengine import Document, EmbeddedDocumentField
 from mongoengine.base import TopLevelDocumentMetaclass
@@ -8,16 +8,18 @@ from aggify.utilty import get_db_field
 
 
 class Operators:
+    # noinspection SpellCheckingInspection
     QUERY_OPERATORS = {
         "exact": "$eq",
-        "iexact": "$eq",
+        "iexact": "$regex",
         "contains": "$regex",
-        "icontains": "$regex",  # noqa
+        "icontains": "$regex",
         "startswith": "$regex",
-        "istartswith": "$regex",  # noqa
+        "istartswith": "$regex",
         "endswith": "$regex",
-        "iendswith": "$regex",  # noqa
+        "iendswith": "$regex",
         "in": "$in",
+        "nin": "$nin",
         "ne": "$ne",
         "not": "$not",
     }
@@ -34,7 +36,26 @@ class Operators:
         **COMPARISON_OPERATORS,
     }
 
-    def __init__(self, match_query: dict[str, Any]):
+    # noinspection SpellCheckingInspection
+    REGEX_PATTERNS = {
+        "iexact": "^{value}$",
+        "contains": "{value}",
+        "icontains": "{value}",
+        "startswith": "^{value}",
+        "istartswith": "^{value}",
+        "endswith": "{value}$",
+        "iendswith": "{value}$",
+    }
+
+    # noinspection SpellCheckingInspection
+    REGEX_OPTIONS = {
+        "iexact": "i",
+        "icontains": "i",
+        "istartswith": "i",
+        "iendswith": "i",
+    }
+
+    def __init__(self, match_query: Dict[str, Any]):
         self.match_query = match_query
 
     def compile_match(self, operator: str, value, field: str):
@@ -42,41 +63,31 @@ class Operators:
         # I think there should be easier way to inject comparison operators to be defined per each
         # like map an existing template to each operator
 
-        if operator in ["exact", "iexact"]:
-            self.match_query[field] = {Operators.ALL_OPERATORS[operator]: value}
+        if operator in Operators.REGEX_PATTERNS:
+            if isinstance(value, F):
+                raise ValueError("Not implemented yet")
+            pattern = Operators.REGEX_PATTERNS[operator].format(value=value)
+            # Create the base query with the pattern
+            query = {Operators.ALL_OPERATORS[operator]: pattern}
 
-        elif operator in [
-            "contains",
-            "startswith",
-            "endswith",
-            "icontains",
-            "istartswith",
-            "iendswith",
-        ]:  # noqa
-            self.match_query[field] = {
-                Operators.ALL_OPERATORS[operator]: f".*{value}.*",
-                "$options": "i",
-            }
+            # If there's an option for the operator, add it to the query
+            if operator in Operators.REGEX_OPTIONS:
+                query["$options"] = Operators.REGEX_OPTIONS[operator]
 
-        elif operator in Operators.ALL_OPERATORS[operator]:
+            self.match_query[field] = query
+        elif operator in Operators.ALL_OPERATORS:
             if isinstance(value, F):
                 self.match_query["$expr"] = {
-                    Operators.ALL_OPERATORS[operator]: [
-                        f"${field}",
-                        value.to_dict(),
-                    ]
+                    Operators.ALL_OPERATORS[operator]: [f"${field}", value.to_dict()]
                 }
-
             else:
                 self.match_query[field] = {Operators.ALL_OPERATORS[operator]: value}
-        else:
-            self.match_query[field] = {Operators.ALL_OPERATORS[operator]: value}
 
         return self.match_query
 
 
 class Q:
-    def __init__(self, pipeline: list | None = None, **conditions):
+    def __init__(self, pipeline: Union[list, None] = None, **conditions):
         pipeline = pipeline or []
         self.conditions: dict[str, list] = (
             Match(
@@ -91,7 +102,7 @@ class Q:
         yield "$match", self.conditions
 
     def __or__(self, other):
-        if self.conditions.get("$or", None):
+        if self.conditions.get("$or"):
             self.conditions["$or"].append(dict(other)["$match"])
             combined_conditions = self.conditions
 
@@ -100,7 +111,7 @@ class Q:
         return Q(**combined_conditions)
 
     def __and__(self, other):
-        if self.conditions.get("$and", None):
+        if self.conditions.get("$and"):
             self.conditions["$and"].append(dict(other)["$match"])
             combined_conditions = self.conditions
         else:
@@ -113,16 +124,16 @@ class Q:
 
 
 class F:
-    def __init__(self, field: str | dict[str, list]):
+    def __init__(self, field: Union[str, Dict[str, list]]):
         if isinstance(field, str):
-            self.field = f"${field}"
+            self.field = f"${field.replace('__', '.')}"
         else:
             self.field = field
 
     def to_dict(self):
         return self.field
 
-    def __add__(self, other):  # TODO: add type for 'other'
+    def __add__(self, other):
         if isinstance(other, F):
             other = other.field
 
@@ -224,13 +235,21 @@ class Cond:
 
 
 class Match:
-    def __init__(self, matches: dict[str, Any], base_model: Type[Document] | None):
+    def __init__(
+        self, matches: Dict[str, Any], base_model: Union[Type[Document], None]
+    ):
         self.matches = matches
         self.base_model = base_model
 
     @staticmethod
     def validate_operator(key: str):
-        if (operator := key.rsplit("__", 1)[1]) not in Operators.COMPARISON_OPERATORS:
+        _op = key.rsplit("__", 1)
+        try:
+            operator = _op[1]
+        except IndexError:
+            raise InvalidOperator(str(_op)) from None
+
+        if operator not in Operators.COMPARISON_OPERATORS:
             raise InvalidOperator(operator)
 
     def is_base_model_field(self, field) -> bool:
@@ -248,23 +267,23 @@ class Match:
         """
         return self.base_model is not None and (
             isinstance(
-                self.base_model._fields.get(field),
+                self.base_model._fields.get(field),  # noqa
                 (EmbeddedDocumentField, TopLevelDocumentMetaclass),
-            )  # noqa
+            )
         )
 
-    def compile(self, pipelines: list) -> dict[str, dict[str, list]]:
+    def compile(self, pipelines: list) -> Dict[str, Dict[str, list]]:
         match_query = {}
         for key, value in self.matches.items():
+            if isinstance(value, F):
+                if F.is_suitable_for_match(key) is False:
+                    raise InvalidOperator(key)
+
             if "__" not in key:
                 key = get_db_field(self.base_model, key)
                 match_query[key] = value
                 continue
 
-            if isinstance(value, F):
-                if F.is_suitable_for_match(key) is False:
-                    raise InvalidOperator(key)
-
             field, operator, *_ = key.split("__")
             if (
                 self.is_base_model_field(field)
