@@ -1,22 +1,11 @@
-from typing import Any, Type
+from typing import Any, Union, List, Dict
 
 from mongoengine import Document
-
+from aggify.types import CollectionType
 from aggify.exceptions import MongoIndexError, InvalidField, AlreadyExistsField
 
 
-def int_to_slice(final_index: int) -> slice:
-    """
-    Converts an integer to a slice, assuming that the start index is 0.
-
-    Examples:
-        >>> int_to_slice(3)
-        slice(0, 2)
-    """
-    return slice(0, final_index)
-
-
-def to_mongo_positive_index(index: int | slice) -> slice:
+def to_mongo_positive_index(index: Union[int, slice]) -> slice:
     if isinstance(index, int):
         if index < 0:
             raise MongoIndexError
@@ -33,7 +22,7 @@ def to_mongo_positive_index(index: int | slice) -> slice:
     return index
 
 
-def check_fields_exist(model: Document, fields_to_check: list[str]) -> None:
+def check_fields_exist(model: CollectionType, fields_to_check: List[str]) -> None:
     """
     Check if the specified fields exist in a model's fields.
 
@@ -45,8 +34,8 @@ def check_fields_exist(model: Document, fields_to_check: list[str]) -> None:
         InvalidField: If any of the specified fields are missing in the model's fields.
     """
     missing_fields = [
-        field for field in fields_to_check if not model._fields.get(field)
-    ]  # noqa
+        field for field in fields_to_check if not model._fields.get(field)  # noqa
+    ]
     if missing_fields:
         raise InvalidField(field=missing_fields[0])
 
@@ -79,18 +68,18 @@ def replace_values_recursive(obj, replacements):
 
 
 def convert_match_query(
-    d: dict,
-) -> dict[Any, list[str | Any] | dict] | list[dict] | dict:
+    d: Dict,
+) -> Union[Dict[Any, Union[List[Union[str, Any]], Dict]], List[Dict], Dict]:
     """
-    Recursively transform a dictionary to modify the structure of '$eq' and '$ne' operators.
+    Recursively transform a dictionary to modify the structure of operators.
 
     Args:
     d (dict or any): The input dictionary to be transformed.
 
     Returns:
-    dict or any: The transformed dictionary with '$eq' and '$ne' operators modified.
+    dict or any: The transformed dictionary with operators modified.
 
-    This function recursively processes the input dictionary, looking for '$eq' and '$ne' operators
+    This function recursively processes the input dictionary, looking for operators
     within sub-dictionaries. When found, it restructures the data into the format {'$eq' or '$ne': [field, value]}.
     For other fields, it processes them recursively to maintain the dictionary structure.
 
@@ -103,7 +92,8 @@ def convert_match_query(
     if isinstance(d, dict):
         new_dict = {}
         for key, value in d.items():
-            if isinstance(value, dict) and ("$eq" in value or "$ne" in value):
+            operators = {"$eq", "$ne", "$gt", "$lt", "$gte", "lte"}
+            if isinstance(value, dict) and any(op in value for op in operators):
                 for operator, operand in value.items():
                     new_dict[operator] = [f"${key}", operand]
             else:
@@ -115,7 +105,7 @@ def convert_match_query(
         return d
 
 
-def check_field_exists(model: Type[Document], field: str) -> None:
+def check_field_exists(model: CollectionType, field: str) -> None:
     """
     Check if a field exists in the given model.
 
@@ -130,7 +120,7 @@ def check_field_exists(model: Type[Document], field: str) -> None:
         raise AlreadyExistsField(field=field)
 
 
-def get_db_field(model: Type[Document], field: str, add_dollar_sign=False) -> str:
+def get_db_field(model: CollectionType, field: str, add_dollar_sign=False) -> str:
     """
     Get the database field name for a given field in the model.
 
